#!/usr/bin/env ruby

######################################################################
# cut    : VNW VPW port of IO pad cell
# remove : IO pad filler cell, corner cell
#   usage: ./cut_vnw_vpw.rb <input.rb> <output.rb>
######################################################################

######################################################################
# setting
######################################################################
IOPAD  = 0
SPACER = 1

table = {}
table["IOCB2EITNNMXA02"] = IOPAD
table["IOCB2EOT2X2LA02"] = IOPAD
table["IOCB2EPD5PI11"]   = IOPAD
table["IOCB2EPG5PB11"]   = IOPAD
table["IOCB2EPE5PE11"]   = IOPAD
table["IOCB2EWSAOC0A1"]  = IOPAD
table["IOCB2EWSAOC0A1"]  = IOPAD
table["IOCB2EWDAOC0A1"]  = IOPAD
table["IOCB2EWGAOC0A1"]  = IOPAD
table["IOCB2EW3AOC0A1"]  = IOPAD
table["ZCGCB2E4C0XXA1"]  = IOPAD

table["IOCB2ESB4C0A1"]   = SPACER
table["IOCB2ES14C0A1"]   = SPACER
table["IOCB2ES24C0A1"]   = SPACER
table["IOCB2ES54C0A1"]   = SPACER
table["IOCB2ESA4C0A1"]   = SPACER
table["IOCB2ESD4C0A1"]   = SPACER
table["IOCB2ESE4C0A1"]   = SPACER
table["IOCB2DA2E4C01"]   = SPACER
table["IOCB2DB2E4C01"]   = SPACER
table["IOCB2DC2E4C01"]   = SPACER

######################################################################
# argument check
######################################################################
if (ARGV.size < 2) then
  puts("cut    : VNW VPW port of IO pad cell")
  puts("remove : IO pad filler cell, corner cell")
  puts("  usage: ./cut_vnw_vpw.rb <input.rb> <output.rb>")
  exit
end

fd = open(ARGV[1], "w")

######################################################################
# read
######################################################################
flag_replace = false
flag_cut     = false
output       = ""

open(ARGV[0]).each do |line|
  # ignore empty line ############################
  if (line =~ /^ *$/) then
    flag_replace = false
    flag_cut     = false
    fd.puts(output)
    next
  end

  # token check ##################################
  tokens = line.split()
  if (tokens.length < 2) then
    fd.puts(line)
    next
  end
  ret    = table[tokens[1]]

  # check ########################################
  if (ret == IOPAD) then
    if (flag_replace || flag_cut) then
      fd.puts(output)
    end
    flag_replace = true
    flag_cut     = false
    output       = tokens[0]
    tokens.shift()
  elsif (ret == SPACER) then
    if (flag_replace || flag_cut) then
      fd.puts(output)
    end
    flag_replace = false
    flag_cut     = true
    output       = ""
  elsif (tokens[0] != "+") then
    flag_replace = false
    flag_cut     = false
  end
 
  # IOPAD ########################################
  if (flag_replace) then
    tokens.each do |token|
      if (token !~ /VNW=/ && token !~ /VPW=/ && token != "+") then
        output += sprintf(" %s", token)
      end
    end

  # SPACER #######################################
  elsif (flag_cut) then
    output = ""

  # normal #######################################
  else
    fd.puts(line)
  end

#  if (flag_replace || flag_cut) then
#    puts("-----------------------------------------")
#    puts(line)
#    puts(ret, flag_replace, flag_cut)
#   puts(output)
#  end
end
