#!/usr/bin/env ruby

# returns false if file unreadable
def get_lines(filename)
  if not (File.readable?(filename) and File.ftype(filename) == 'file')
    warn 'WARNING : "' + filename + '" unreadable'
    return []
  end
  
  warn 'INFO    : Reading "' + filename + '"...'
  src_lines = Array.new
  open(filename).each do |line|
    src_lines << line.chomp
  end
  
  return src_lines
end


# cdl_stack (Array of String) = for preventing recursive include
def include_file(lines, cdl_stack, basedir)
  del_line_no = Array.new  # for lines .include "hogehoge.cdl"
  
  lines.each_index do |i|
    if lines[i] =~ /^\.include\s+"([\w\.\/]+)"/i
      filename  = basedir + '/' + $1
      real_path_filename = File.expand_path(filename)
      if cdl_stack.include? real_path_filename
        warn 'WARNING : Tried to read "' + filename + '" recursively, and was ignored'
        del_line_no << i
      else
        cdl_stack << real_path_filename
        basedir = basedir + '/' + File.dirname(filename)
        inc_lines = get_lines(filename)
        inc_lines = include_file(inc_lines, cdl_stack, basedir)
        lines[i] = inc_lines
      end
    end
  end
  
  del_line_no.reverse!
  del_line_no.each do |i|
    lines.delete_at(i)
  end
  
  return lines.flatten
end



# lines     (Array of String) = lines in source cdl file
# del_cells (Array of String) = cell names
def del_insts(lines, del_cells)
  cur_cell    = nil
  del         = false
  del_line_no = Array.new
  
  lines.each_index do |i|
    case lines[i]
    when /^X/
      tokens    = lines[i].split(/\s+/)
      inst_name = tokens.shift
      cur_cell  = tokens.shift
      if del_cells.include? cur_cell
        del = true
        warn 'INFO    : Deleting : ' + inst_name + '(' + cur_cell + ')'
        del_line_no << i
      end
    when /^\+/
      if del
        del_line_no << i
      end
    else
      del = false
    end
  end
  
  del_line_no.reverse!
  del_line_no.each do |i|
    lines.delete_at(i)
  end
  
  return lines
end


def cat_tail(lines, adds)
  adds.each do |line|
    warn 'INFO    : Adding at the end : ' + add
    lines << add
  end
  
  return lines
end


# del_regexps (Array of Regexp)
def del_lines(lines, del_regexps)
  del_line_no = Array.new
  
  lines.each_index do |i|
    del_regexps.each do |del_regexp|
      if lines[i] =~ del_regexp
        warn 'INFO    : Deleting : ' + lines[i]
        del_line_no << i
        break
      end
    end
  end
  
  del_line_no.reverse!
  del_line_no.each do |i|
    lines.delete_at(i)
  end
  
  return lines
end


# cell_regexp (Regexp)    = regexp for cell master name
# ports (Array of String) = port names
def del_inst_port(lines, cell_regexp, ports)
  cur_cell = nil
  del      = false
  
  lines.each_index do |i|
    case lines[i]
    when /^X/
      tokens = lines[i].split(/\s+/)
      cur_cell = tokens[i]
      if cur_cell =~ cell_regexp
        ports.each do |port|
          lines[i].gsub!(/#{port}\s*=\*\w+/, '')
        end
      end
    when /^\+/
      if del
        ports.each do |port|
          lines[i].gsub!(/#{port}\s*=\s*\w+/, '')
        end
      end
    else
      del = false
    end
  end
  
  return lines
end
