#!/usr/bin/env ruby

################################################################
# SCRIPT : cdl__mod_for_tope_module_lvs.rb
################################################################

USAGE = 'USAGE   : ' + $0 + ' SRC.cdl DST.cdl [MODIFY_FROM(smash/del_inst/del_well/dump)]'

CORNER_CELL       = 'ZCGCB2E4C0XXA1'

PAD_FILLER_CELL   = 'IOCB2ES'
# PAD_FILLER_CELL = 'IOCB2ES||D4C0A1'
# PAD_FILLER_CELL = 'IOCB2ES||A4C0A1'
# PAD_FILLER_CELL = 'IOCB2ES||54C0A1'
# PAD_FILLER_CELL = 'IOCB2ES||24C0A1'
# PAD_FILLER_CELL = 'IOCB2ES||14C0A1'

INPUT_WIRE_CELL   = 'WSAOC0A1'
OUTPUT_WIRE_CELL  = 'WSAOC0A1'
VDD_WIRE_CELL     = 'WDAOC0A1'
VDE_WIRE_CELL     = 'W3AOC0A1'
VSS_WIRE_CELL     = 'WGAOC0A1'

INPUT_PAD_CELL    = 'ITNNMXA02'
OUTPUT_PAD_CELL   = 'OT2X2LA02'
VDD_PAD_CELL      = 'D5PI11'
VDE_PAD_CELL      = 'G5PB11'
VSS_PAD_CELL      = 'E5PE11'
MEMORY_CELL       = 'RAM'
DIVIDER_CELL      = 'DC2E4C01'

# INPUT_WIRE_CELL  = 'IOCB2E||WSAOC0A1'
# OUTPUT_WIRE_CELL = 'IOCB2E||WSAOC0A1'
# VDD_WIRE_CELL    = 'IOCB2E||WDAOC0A1'
# VDE_WIRE_CELL    = 'IOCB2E||W3AOC0A1'
# VSS_WIRE_CELL    = 'IOCB2E||WGAOC0A1'

# INPUT_PAD       = 'IOCB2E||ITNNMXA02'
# OUTPUT_PAD      = 'IOCB2E||OT2X2LA02'
# VDD_PAD_CELL    = 'IOCB2EP||D5PI11'
# VDE_PAD_CELL    = 'IOCB2EP||G5PB11'
# VSS_PAD_CELL    = 'IOCB2EP||E5PE11'
# DIVIDER_CELL    = 'IOCB2E||DC2E4C01'


VPW_CONNECTION    = / VPW=\S+/
VNW_CONNECTION    = / VNW=\S+/


######## COMMAND LINE ##########################################
if ARGV.size < 2
  abort USAGE
end
src_file     = ARGV.shift
smash_file   = src_file + '__1_smashed'
del_file     = src_file + '__2_del_inst'
no_well_file = src_file + '__3_no_well'
dst_file     = ARGV.shift
modify_from  = ARGV.shift

case modify_from
when 'smash', nil
  modify_from = 0
when 'del_inst'
  modify_from = 1
when 'del_well'
  modify_from = 2
when 'dump'
  modify_from =3
else
  abort USAGE
end


######## SMASH #################################################
print 'INFO    : ' + `date`; STDOUT.flush

if modify_from == 0
  print 'INFO    : Smashing...'; STDOUT.flush
  tmp1 = open(smash_file, 'w')
  open(src_file).each do |line|
    line.chomp!
    if line[0,1] == '+'
      line['+'] = ''
      tmp1.print line
    else
      tmp1.print "\n"
      tmp1.print line
    end
  end
  tmp1.print "\n"
  print "done\n"
  print 'INFO    : ' + `date`; STDOUT.flush
  tmp1.close
end


######## DELETE CORNER & FILLER CELL ###########################
if modify_from <= 1
  print 'INFO    : Deleting corner and filler cells...'; STDOUT.flush
  `grep -v #{PAD_FILLER_CELL} #{smash_file} | grep -v #{CORNER_CELL} > #{del_file}`
#  tmp2 = open(del_file, 'w')
#  open(smash_file).each do |line|
#    if line[0,1] == 'X'
#      cell_name = line.split[1]
#      next if (cell_name[0,7] == PAD_FILLER_CELL) or (cell_name == CORNER_CELL)
#    end
#    tmp2.print line
#  end
#  tmp2.close
  print "done\n"
  print 'INFO    : ' + `date`; STDOUT.flush
end


######## DELETE WELL FROM IO CELLS #############################
if modify_from <= 2
  print 'INFO    : Deleting VNW/VPW...'; STDOUT.flush
  dst = open(dst_file, 'w')
  open(del_file).each do |line|
    if line[0,1] == 'X'
      cell_name = line.split[1]
      if (cell_name[6,8] == INPUT_WIRE_CELL) or (cell_name[6,8] == OUTPUT_WIRE_CELL) or
          (cell_name[6,8] == VDD_WIRE_CELL) or (cell_name[6,8] == VDE_WIRE_CELL) or (cell_name[6,8] == VSS_WIRE_CELL) or
          (cell_name[6,9] == INPUT_PAD_CELL) or (cell_name[6,9] == OUTPUT_PAD_CELL) or
          (cell_name[7,6] == VDD_PAD_CELL) or (cell_name[7,6] == VDE_PAD_CELL) or (cell_name[7,6] == VSS_PAD_CELL) or
          (cell_name[0,3] == MEMORY_CELL) or (cell_name[6,8] == DIVIDER_CELL)
        dst.print line.sub(VPW_CONNECTION, '').sub(VNW_CONNECTION, '')
      else
        dst.print line
      end
    else
      dst.print line
    end
  end
  dst.close
  print "done\n"
  print 'INFO    : ' + `date`; STDOUT.flush
end
